;
;                WARAJEVO ZX Spectrum emulator, Release 2.5
;                     (C) Zeljko Juric & Samir Ribic
;
;   Plug-in for the emulation of the Kempston mouse for ZX Spectrum
;

;
; External driver must define handlers for UserOut (via int 249) and UserIn
; (via int 248). COM type of driver is recommended but this is not necessary.
;

CSeg       segment

           assume  cs:CSeg

           org     100h                  ; COM must start at 100H

;
; A Kempston mouse is simple device. It uses port FBDFh and FFDFh for reading
; x and y coordinated of the mouse, and FADFh for reading the state of the
; mouse buttons (three buttons).
;

Start:     jmp     Init

OldUserIn  dw      Dummy,?                  ; Vector for old user input
XPos       dw      0                        ; X coordinate
YPos       dw      0                        ; Y coordinate

;
; "User in" interface
;

           db      'USER'                   ; A signature is necessary

UserIn:    cmp     cl,0DFh                  ; Test for port address, and exit
           je      InOK                     ;  via old handler if it is not
           jmp     dword ptr OldUserIn      ;  Centronics port

;
; Now, Kempston mouse port is recognized.
;

InOK:      push    cx
           mov     ax,12                    ; Disable intercepting of right
           xor     cx,cx                    ;  mouse button pressing
           int     33h
           mov     ax,3                     ; Read mouse coords and buttons
           int     33h
           mov     ax,11
           int     33h
           add     XPos,cx
           add     YPos,dx
           pop     ax
           cmp     ah,0FBh                  ; X-coordinate port
           je      XCoord
           cmp     ah,0FFH                  ; Y-coordinate port
           je      YCoord
           cmp     ah,0FAh                  ; Buttons port
           mov     ah,255
           je      Buttons
Done:      stc                              ; Signal by CF that I/O address is
           retf    2                        ;  recognized

Buttons:   and     bl,7                     ; The bits returned by int 33 is
           mov     cl,bl                    ;  not in order needed for Kempston
           and     cl,1                     ;  mouse, so reordering of bits is
           rol     cl,1                     ;  necessary
           mov     dl,bl
           and     cl,2
           ror     cl,1
           and     bl,4
           or      bl,cl
           or      bl,dl
           not     bl
           mov     ah,bl
           jmp     Done

XCoord:    mov     ah,byte ptr XPos         ; Pick X coordinate
           jmp     Done

YCoord:    mov     ah,byte ptr YPos         ; Pick Y coordinate
           neg     ah                       ; Y axis is reversed on Spectrum
           jmp     Done

;
; Dummy interrupt routine, if the old handlers are not found.
;

Dummy:     iret

;
; Effective start of the program.
;

Init:      mov     OldUserIn[2],cs          ; Make the dummy vectors complete
           lea     dx,NameText              
           mov     ah,9                     ; Display the copyright
           int     21h
           mov     ax,35F8h                 ; Check for presence of other
           int     21h                      ;  plug-ins
           cmp     bx,4
           jb      Install
           cmp     word ptr es:[bx-4],'SU'  ; Check for signatures
           jne     Install
           cmp     word ptr es:[bx-2],'RE'
           je      Present
Install:   mov     ax,25F8h
           lea     dx,UserIn
           int     21h
           lea     dx,ResLdText             ; Yet another short message
           mov     ah,9
           int     21h
           lea     dx,Init+1                ; Terminate and stay resident
           int     27h

Present:   mov     OldUserIn,bx             ; Store link to other plug-ins
           mov     OldUserIn[2],es
           lea     dx,WarnText              ; Already installed
           mov     ah,9
           int     21h
           jmp     Install

NameText   db      'Kempston mouse support plug-in for Warajevo ZX Spectrum '
           db      'emulator.',13,10,'$'
ResLdText  db      'Resident part installed.',13,10,'$'
WarnText   db      'Warning: another plug-in already installed!',13,10,'$'

CSeg       ends

           end     Start

